#include <sqlextract/filter/filter_core.h>
#include <sqlextract/filter/filter_logic.h>
#include <sqlextract/util.h>
#include <sqlextract/filter/arbval.h>
#include <stdlib.h>
#include <stdarg.h>

static void filter_multi_destroy(void *storage) {
	struct filter_t **cur;
	for (cur = (struct filter_t **) storage; *cur; cur++) {
		filter_destroy(*cur);
	}
	free(storage);
}

static struct filter_t *filter_multi_create(
		void (*process)(struct arbval_t *, void *, void *), int count,
		va_list ap) {
	struct filter_t **subfilters;
	int i;

	subfilters = lf_calloc(count + 1, sizeof(struct filter_t *));

	for (i = 0; i < count; i++) {
		subfilters[i] = va_arg(ap, struct filter_t *);
	}
	subfilters[count] = NULL;

	return filter_create(process, filter_multi_destroy, (void*) subfilters);
}

static void filter_process_and(struct arbval_t *result, void *object,
		void *storage) {
	struct filter_t **cur;
	struct arbval_t *subf_result;
	arbval_typecast_integer(result);
	for (cur = (struct filter_t **) storage; *cur; cur++) {
		subf_result = filter_process(*cur, object);
		if (arbval_is_false(subf_result)) {
			result->value.integer = 0;
			return;
		}
	}
	result->value.integer = 1;
}

struct filter_t *filter_create_and(int count, ...) {
	va_list ap;
	struct filter_t *flt;
	va_start(ap, count);
	flt = filter_multi_create(filter_process_and, count, ap);
	va_end(ap);
	return flt;
}

static void filter_process_or(struct arbval_t *result, void *object,
		void *storage) {
	struct filter_t **cur;
	struct arbval_t *subf_result;
	arbval_typecast_integer(result);
	for (cur = (struct filter_t **) storage; *cur; cur++) {
		subf_result = filter_process(*cur, object);
		if (arbval_is_true(subf_result)) {
			result->value.integer = 1;
			return;
		}
	}
	result->value.integer = 0;
}

struct filter_t *filter_create_or(int count, ...) {
	va_list ap;
	struct filter_t *flt;
	va_start(ap, count);
	flt = filter_multi_create(filter_process_or, count, ap);
	va_end(ap);
	return flt;
}

static void filter_process_not(struct arbval_t *result, void *object,
		void *storage) {
	struct filter_t *subf = (struct filter_t *) storage;
	arbval_typecast_integer(result);
	result->value.integer = arbval_is_false(filter_process(subf, object));
}

static void filter_destroy_not(void *storage) {
	struct filter_t *subf = (struct filter_t *) storage;
	filter_destroy(subf);
}

struct filter_t *filter_create_not(struct filter_t *subfilter) {
	return filter_create(filter_process_not, filter_destroy_not,
			(struct filter_t *) subfilter);
}
